@@ -15,7 +15,7 @@ import abc
 import logging
 import tempfile
 from os import listdir
-from os.path import isfile, join
+from os.path import isfile, join, basename
 
 
 class Clouds(abc.ABC):
@@ -48,6 +48,7 @@ class S3(Clouds):
 
     def upload_directory(self, directory, **options):
         bucket_name = options.pop('bucket_name', None)
+        folder = options.pop('folder', '')
         if bucket_name is None:
             raise ValueError('You should pass a bucket name')
 
@@ -55,7 +56,7 @@ class S3(Clouds):
 
         try:
             for file in files:
-                self.s3.upload_file(join(directory, file), bucket_name, file)
+                self.s3.upload_file(join(directory, file), bucket_name, join(folder, file))
         except self.err as e:
             logging.error(e)
             raise RuntimeError(e)
@@ -70,7 +71,7 @@ class S3(Clouds):
             raise ValueError('You should pass a bucket and key name')
 
         if filename is None:
-            filename = tempfile.NamedTemporaryFile(suffix='_' + key + '_py_ff_vi_st.tmp', delete=False)
+            filename = tempfile.NamedTemporaryFile(prefix=basename(key), delete=False)
         else:
             filename = open(filename, 'wb')
 
@@ -116,15 +117,15 @@ class GCS(Clouds):
             raise ValueError('You should pass a bucket name')
 
         bucket = self.client.get_bucket(bucket_name)
-
-        if filename is None:
-            with tempfile.NamedTemporaryFile(suffix='_py_ff_vi_st.tmp', delete=False) as tmp:
-                filename = tmp.name
-
         object_name = options.pop('object_name', None)
+
         if object_name is None:
             raise ValueError('You should pass an object name')
 
+        if filename is None:
+            with tempfile.NamedTemporaryFile(prefix=basename(object_name), delete=False) as tmp:
+                filename = tmp.name
+
         blob = bucket.get_blob(object_name, options)
         blob.download_to_filename(filename)
 
@@ -166,7 +167,7 @@ class MAS(Clouds):
             raise ValueError('You should pass a container name and a blob name')
 
         if filename is None:
-            with tempfile.NamedTemporaryFile(suffix='_py_ff_vi_st.tmp', delete=False) as tmp:
+            with tempfile.NamedTemporaryFile(prefix=basename(blob), delete=False) as tmp:
                 filename = tmp.name
 
         try:
@@ -181,10 +182,11 @@ class MAS(Clouds):
 
 
 class CloudManager:
-    def __init__(self):
+    def __init__(self, filename: str = None):
         """
         @TODO: add documentation
         """
+        self.filename = filename
         self.clouds = []
 
     def add(self, cloud: Clouds, **options):
